// JavaScript for FwTopicPermissionsPlugin
// 
// depends on jQuery, e.g. via JQueryPlugin
// tested on Foswiki 1.0.4
// 
// Send bug reports to
// Sven Arweiler, sarweiler@seibert-media.net
// Thomas Zobel, tzobel@seibert-media.net


$(document).ready(function() {
	
	// set default for ALLOW/DENY mode
	var allowdeny = "allow";
	
	// set default for View/Change/Rename
	var viewchangerename = "view";
	
	// set default for User/Group
	var usergroup = "user";
	
	// some clean-up functions
	function clear_vcr() {
		$("#viewchangerename a").css("fontWeight", "normal");
	}
	
	function clear_ug() {
		$("#usergroup a").css("fontWeight", "normal");
	}
	
	function clear_ad() {
		$("#allowdeny a").css("fontWeight", "normal");
	}
	
	function clear_tables() {
		$("#userlist_table").hide();
		$("#grouplist_table").hide();
	}
	
	function clear_all() {
		clear_vcr();
		clear_ug();
	}

	// clicked on View, Change or Rename
	function vcr_clicked(clicked) {
		var new_class = "";
		switch(clicked.attr("id")) {
			case "user_view":
				new_class = ".tp_view";
				break;
			case "user_change":
				new_class = ".tp_change";
				break;
			case "user_rename":
				new_class = ".tp_rename";
				break;
			default:
				new_class = ".tp_view";
				break;
		}
		
		new_class += "_" + allowdeny;
		
		clear_vcr();
		clicked.css("fontWeight", "bold");
		$("#topicpermissions_" + usergroup + "table tr td input").removeAttr("checked");
		$("#topicpermissions_" + usergroup + "table tr td " + new_class).attr("checked", "checked");
	}
	
	function get_checked(permission) {
		var perm_array = new Array;
		var perm_string = new String;
		$("#topicpermissions input:checkbox.tp_" + permission).each(function() {
			perm_array.push($(this).attr("id").replace(/_check/g, ""));
		});
		
		if (perm_array.length > 0) {
			perm_string = perm_array.sort().join(", ");
		} else {
			perm_string = "";
		}
		
		return (perm_string);
	}
	
	
	function save() {
		var view_allow = get_checked("view_allow");
		var change_allow = get_checked("change_allow");
		var rename_allow = get_checked("rename_allow");
		
		var view_deny = get_checked("view_deny");
		var change_deny = get_checked("change_deny");
		var rename_deny = get_checked("rename_deny");
		
		$.post(
			topicPermissionsBinPath + "/rest/FwTopicPermissionsPlugin/savePrefs",
			{
				"web": topicPermissionsWeb,
				"topic": topicPermissionsWeb + "." + topicPermissionsTopic,
				"realtopicname": topicPermissionsTopic,
				"view_allow": view_allow,
				"change_allow": change_allow,
				"rename_allow": rename_allow,
				"view_deny": view_deny,
				"change_deny": change_deny,
				"rename_deny": rename_deny
			},
			function() {
				$("#tp_save_button").val("Done");
				window.setTimeout(function() {$("#tp_save_button").val("Save");}, 2000);
			}
		);
	}

	// insert HTML into DOM
	$(".patternViewPage .patternTopicActions .patternTopicAction .patternActionButtons").append("<span class=\"foswikiSeparator\">&nbsp;| </span><span class=\"foswikiRequiresChangePermission\"><a id=\"topicpermissions_link\" title=\"Topic Permissions\" rel=\"nofollow\" href=\"javascript: void(0);\">Topic permissions</a></span>");
	
	$(".patternTopicActions").after("<div id=\"topicpermissions\"><strong>Topic Permissions</strong><br><div id=\"usergroup\"><a id=\"remove_all_user_permissions\" href=\"javascript: void(0);\">Remove all user permissions</a> | <a id=\"remove_all_group_permissions\" href=\"javascript: void(0);\">Remove all group permissions</a><br /><a id=\"change_user_permissions\" href=\"javascript: void(0);\">Change user permissions</a> | <a id=\"change_group_permissions\" href=\"javascript: void(0);\">Change group permissions</a></div><div id=\"tp_filter\">Filter: <input id=\"filter_users\" type=\"text\" /></div><div id=\"allowdeny\"><a id=\"user_allow\" href=\"javascript: void(0);\">ALLOW</a> | <a id=\"user_deny\" href=\"javascript: void(0);\">DENY</a></div><div id=\"viewchangerename\"><a id=\"user_view\" href=\"javascript: void(0);\">View</a> | <a id=\"user_change\" href=\"javascript: void(0);\">Change</a> | <a id=\"user_rename\" href=\"javascript: void(0);\">Rename</a></div><div id=\"selectall\"></div><div id=\"userlist_table\"></div><div id=\"grouplist_table\"></div></div>");
	
	if (saveButtonMode) {
		$("#grouplist_table").after("<input id='tp_save_button' type='submit' value='Save' />");
	}
	
	
	
	// Topic permissions
	$("#topicpermissions_link").toggle(
		function() {
			$("#userlist_table").load(
				topicPermissionsBinPath + "/rest/FwTopicPermissionsPlugin/getUserList",
				{"web":topicPermissionsWeb, "topic":topicPermissionsWeb + "." + topicPermissionsTopic, "mode":"user"},
				function() {
					$("#change_user_permissions").trigger("click");
					$("#user_allow").trigger("click");
					$("#user_view").trigger("click");
				}
			);
			$("#grouplist_table").load(
				topicPermissionsBinPath + "/rest/FwTopicPermissionsPlugin/getUserList",
				{"web":topicPermissionsWeb, "topic":topicPermissionsWeb + "." + topicPermissionsTopic, "mode":"group"},
				function() {
					$("#topicpermissions input:checkbox").click(function() {
						var current_class = $(this).attr("class");
						
						if (exclusiveMode) {
							var notallowdeny = new String;
							
							if (allowdeny == "allow") {
								notallowdeny = "deny";
							} else {
								notallowdeny = "allow";
							}
							
							if ($(this).hasClass("tp_" + viewchangerename + "_" + allowdeny)) {
								$(this).removeClass("tp_" + viewchangerename + "_" + allowdeny);
								$(this).addClass("tp_" + viewchangerename + "_" + notallowdeny);
							} else {
								$(this).addClass("tp_" + viewchangerename + "_" + allowdeny);
								$("#" + usergroup + "list_table input:checkbox").each(function() {
									var all_checked = get_checked(viewchangerename + "_" + allowdeny);
									var current_id = $(this).attr("id").replace(/_check/,"");
									var id_reg = new RegExp(current_id);
									if (!(id_reg.test(all_checked))) {
										$(this).addClass("tp_" + viewchangerename + "_" + notallowdeny);
									}
								});
								$(this).removeClass("tp_" + viewchangerename + "_" + notallowdeny);
							}
						} else {
							if ($(this).hasClass("tp_" + viewchangerename + "_" + allowdeny)) {
								$(this).removeClass("tp_" + viewchangerename + "_" + allowdeny);
							} else {
								$(this).attr("class", current_class + " tp_" + viewchangerename + "_" + allowdeny);
							}
						}
						
						if (!(saveButtonMode)) {
							save();
						}
					});
				}
			);
			$("#topicpermissions").slideDown();
			
		},
		function() {
			$("#topicpermissions").slideUp();
			$("#filter_users").val("");
		}
	);
	
	// Remove all user permissions
	$("#remove_all_user_permissions").click(function() {
		$("#topicpermissions #userlist_table input:checkbox").each(function() {
			$(this).removeAttr("class");
			$(this).removeAttr("checked");
			if (!(saveButtonMode)) {
				save();
			}
		});
	});
	
	// Remove all group permissions
	$("#remove_all_group_permissions").click(function() {
		$("#topicpermissions #grouplist_table input:checkbox").each(function() {
			$(this).removeAttr("class");
			$(this).removeAttr("checked");
			if (!(saveButtonMode)) {
				save();
			}
		});
	});

	// Change user permissions
	$("#change_user_permissions").click(function() {
		clear_all();
		$(this).css("fontWeight", "bold");
		clear_tables();
		usergroup = "user";
		$("#userlist_table").show();
		$("#user_" + viewchangerename).trigger("click");
	});
	
	// Change group permissions
	$("#change_group_permissions").click(function() {
		clear_all();
		$(this).css("fontWeight", "bold");
		clear_tables();
		usergroup = "group";
		$("#grouplist_table").show();
		$("#user_" + viewchangerename).trigger("click");
	});
	
	
	// ALLOW
	$("#user_allow").click(function() {
		clear_ad();
		$(this).css("fontWeight", "bold");
		allowdeny = "allow";
		$("#user_" + viewchangerename).trigger("click");
	});
	
	// DENY
	$("#user_deny").click(function() {
		clear_ad();
		$(this).css("fontWeight", "bold");
		allowdeny = "deny";
		$("#user_" + viewchangerename).trigger("click");
	});
	
	// View
	$("#user_view").click(function() {
		viewchangerename = "view";
		vcr_clicked($(this));
	});
	
	// Change
	$("#user_change").click(function() {
		viewchangerename = "change";
		vcr_clicked($(this));
	});
	
	// Rename
	$("#user_rename").click(function() {
		viewchangerename = "rename";
		vcr_clicked($(this));
	});
	
	
	// Save-Button
	$("#tp_save_button").click(function() {
		if (saveButtonMode) {
			$(this).val("Saving ...");
			save();
		}
	});
	
	// Filter
	var filter = "";
	$("#filter_users").keyup(function() {
		filter = $(this).val();
		if (filter.length > 0) {
			$("#topicpermissions_usertable tr").hide().filter(function() {
				var filter_reg = new RegExp(filter, "i");
				var row_id = ($(this).attr("id"));
				return (filter_reg.test(row_id));
			}).show();
		} else {
			$("#topicpermissions_usertable tr").show();
		}
	});
	
});